﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TiledSharp;

namespace tmxParser
{
    public class Program
    {
       

        static void Main(string[] args)
        {
          //List<Creature> heroesList = new List<Creature>();
          //List<Creature> enemiesList = new List<Creature>();
          //List<Trigger> triggersList = new List<Trigger>();


            //our tilemap
            byte[] tileMap = new byte[32 * 16];
            byte[] tileMapBackground = new byte[32 * 16];

            //level objects 4 bytes per object
            //0 - hero
            //1 - minXtile maxXtile minYtile maxYtile
            //2 - key
            //3 - door
            //4 - scroll
            //2,3,4,5 
            byte[] mapObjects = new byte[4*6];

            //guns objects
            byte[] eGunsObjects = new byte[4 * 8]; // 4 guns max

      
            Console.WriteLine("TMX Parser by Denis Grachev");
            Console.WriteLine("===========================");
            
            TmxMap map = new TmxMap(args[0]);

            //args[0] = "map01.tmx";

            //int totalTiles = map.Tilesets[0].Tiles.Count();       

            Console.WriteLine("Parsing objects!");

            Console.WriteLine("Parsing tilemap!");

            int minX = 99999;
            int minY = 99999;
            int maxX = 0;
            int maxY = 0;

            int TILE_KEY = 105;            
            int TILE_SCROLL = 113;
            int TILE_DOOR = 30;
            int TILE_PIKE = 117;
            const byte TILE_STAR = 93;

            const byte eGunRight = 18;
            const byte eGunLeft = 19;
            int startAddr = 0;
            int endAddr = 0;


            int currentEGun = 0;

            //import tiles from level layer
            for (int j = 0; j < 16; j++)
            {
                for (int i = 0; i < 32; i++)
                {
                    int tile = (map.TileLayers["level"].Tiles[i + j * 32].Gid);

                    //and skip zeros
                    if (tile > 0)
                    {
                        tile--;
                        tileMap[i + j * 32] = (byte)(tile*2);

                        if (j > maxY)
                            maxY = j;

                        if (j < minY)
                            minY = j;

                        if (i > maxX)
                            maxX = i;

                        if (i < minX)
                            minX = i;


                        //has pikes flag
                        if (tile == TILE_PIKE)
                        {
                            mapObjects[2] = 1;
                        }

                        if (tile == TILE_SCROLL)
                        {
                            mapObjects[4 * 4] = (byte)(i * 8);
                            mapObjects[4 * 4 + 1] = (byte)(j * 8);
                            //addr is bg map
                            int scrollAddr = 32768 - 512 + i + j * 32;

                            mapObjects[4 * 4 + 2] = (byte)(scrollAddr % 256); // low byte
                            mapObjects[4 * 4 + 3] = (byte)(scrollAddr / 256); //high byte
                        }

                       



                        //LIVE BYTE 0; 0 - disabled
                        //COUNTER BYTE 0; counter - shot on 0 till length
                        //LENGTH BYTE 4; length
                        //LENGTH2               BYTE 8; length * 2
                        //ADDR WORD 00000; add in map where beam stars
                        // ADDR2 WORD 00000; add in map where beam ends

                        if (tile==eGunRight)
                            {                                
                                eGunsObjects[currentEGun * 8+1] = (byte)(63+currentEGun*64);//counter
                                startAddr = 32768 + 32 * j + i+1;
                                eGunsObjects[currentEGun * 8 + 4] = (byte)(startAddr % 256);//start addr LOW
                                eGunsObjects[currentEGun * 8 + 5] = (byte)(startAddr / 256);//start addr HIGH
                            }

                        if (tile == eGunLeft)
                        {
                            eGunsObjects[currentEGun * 8] = 1;// enable gun                            
                            endAddr = 32768 + 32 * j + i-1;
                            eGunsObjects[currentEGun * 8 + 6] = (byte)(endAddr % 256);//start addr LOW
                            eGunsObjects[currentEGun * 8 + 7] = (byte)(endAddr / 256);//start addr HIGH
                            int length = endAddr - startAddr+1;
                            eGunsObjects[currentEGun * 8+2] = (byte)(length);// length                            
                            eGunsObjects[currentEGun * 8 + 3] = (byte)(length*2);// length                            
                            currentEGun++;
                            //safe NO MORE THAN 4 EGUNS
                            if (currentEGun > 4)
                                currentEGun = 4;
                        }


                    }
                    else
                    {
                        //copy from BG layer
                        tile = (map.TileLayers["background"].Tiles[i + j * 32].Gid);
                        tile--;
                        tileMap[i + j * 32] = (byte)(tile * 2);
                    }
                }
            }

            //import tiles from level layer
            for (int j = 0; j < 16; j++)
            {
                for (int i = 0; i < 32; i++)
                {
                    int tile = (map.TileLayers["background"].Tiles[i + j * 32].Gid);

                    //and skip zeros
                    if (tile > 0)
                    {
                        tile--;
                        tileMapBackground[i + j * 32] = (byte)(tile * 2);
                    }


                    //has pikes flag
                    if (tile == TILE_PIKE)
                    {
                        mapObjects[2] = 1;
                    }


                    //STAR ON BACKGROUND
                    if (tile == TILE_STAR)
                    {
                        mapObjects[5 * 4] = (byte)(i * 8);
                        mapObjects[5 * 4 + 1] = (byte)(j * 8);
                        //addr is main map
                        int starAddr = 32768 + i + j * 32;

                        mapObjects[5 * 4 + 2] = (byte)(starAddr % 256); // low byte
                        mapObjects[5 * 4 + 3] = (byte)(starAddr / 256); //high byte
                    }

                    //KEY ON BACKGROUND
                    if (tile == TILE_KEY)
                    {
                        mapObjects[2 * 4] = (byte)(i*8);
                        mapObjects[2 * 4 + 1] = (byte)(j * 8);
                        //addr is main map
                        int keyAddr = 32768 + i + j * 32;

                        mapObjects[2 * 4 + 2] = (byte)(keyAddr % 256); // low byte
                        mapObjects[2 * 4 + 3] = (byte)(keyAddr / 256); //high byte
                    }

                    //DOOR ON BACKGROUND
                    if (tile == TILE_DOOR)
                    {
                        mapObjects[3 * 4] = (byte)(i * 8);
                        mapObjects[3 * 4 + 1] = (byte)(j * 8);
                        //addr is main map
                        int doorAddr = 32768 + i + j * 32;

                        mapObjects[3 * 4 + 2] = (byte)(doorAddr % 256); // low byte
                        mapObjects[3 * 4 + 3] = (byte)(doorAddr / 256); //high byte
                    }

                }
            }

            //parse tiles from objects layer
            const byte HERO = 127;

            //enemies 16 bytes per enemy
            byte[] enemies = new byte[4 * 16]; // 4 enemyMax

            //STATE BYTE +0
            //ACTION_COUNTER BYTE +1
            //X BYTE +2
            //Y BYTE +3
            //SPRITE WORD spritesTable + spriteSize * ENEMIES_GLOBAL_SPRITE; attached sprite
            //SCRIPT WORD waitScript; script to run


            //MAP_ADDR        WORD map +8 +9
            //SIN_COUNTER BYTE 0; pointer to sinus Y coord for flying enemies +10
            //TYPE            BYTE ENEMY_FLY +11
            //res           +12
            //EDITOR_HARD_SPRITE_LINK BYTE 0; +13
            //EDITOR_SCRIPT_LINK BYTE 0; +14
            //EDITOR_SPRITE_LINK BYTE 0; +15

            const byte ENEMY_FLY_LEFT = 126;
            const byte ENEMY_FLY_RIGHT = 125;
            const byte ENEMY_WALK_LEFT = 124;
            const byte ENEMY_WALK_RIGHT = 123;
            const byte ENEMY_GHOST = 122;

            const byte ENEMY_FLY = 0;
            const byte ENEMY_WALK = 1;
            const byte ENEMY_GHOSTA = 2;

            

            int enemyCounter = 0;




            //save min max of tiles
            mapObjects[1 * 4] = (byte)(minX * 8);
            mapObjects[1 * 4 + 1] = (byte)(maxX * 8);
            mapObjects[1 * 4 + 2] = (byte)(minY * 8);
            mapObjects[1 * 4 + 3] = (byte)(maxY * 8);

            //parse objects etc
            for (int j = 0; j < 16; j++)
            {
                for (int i = 0; i < 32; i++)
                {
                    int tile = (map.TileLayers["objects"].Tiles[i + j * 32].Gid);

                    //and skip zeros
                    if (tile > 0)
                    {
                        tile--;

                        switch (tile)
                        {
                            case HERO:
                                //HERO params
                                mapObjects[0 * 4] = (byte)(i * 8);//xtart
                                mapObjects[0 * 4+1] = (byte)(j * 8);//ystart
                                break;

                            case ENEMY_GHOST:
                                enemies[enemyCounter * 16 + 0] = 1;//enable
                                enemies[enemyCounter * 16 + 1] = (byte)(enemyCounter * 2);//set start counter 0 2 4 6 
                                enemies[enemyCounter * 16 + 2] = (byte)(i * 8);//x
                                enemies[enemyCounter * 16 + 3] = (byte)(j * 8);//y

                                //sinus pointer for ghost
                                enemies[enemyCounter * 16 + 10] = (byte)(enemyCounter * 64);

                                //type
                                enemies[enemyCounter * 16 + 11] = ENEMY_GHOSTA;

                                //base x
                                enemies[enemyCounter * 16 + 12] = (byte)(i * 8);//x

                                //HARDWARE SPRITE LINK
                                enemies[enemyCounter * 16 + 13] = (byte)(enemyCounter * 2);

                                //SCRIPT LINK
                                enemies[enemyCounter * 16 + 14] = (byte)((126 - tile) * 2 + 8);


                                //step
                                enemyCounter++;
                                if (enemyCounter > 3)
                                    enemyCounter = 0;

                                break;

                            case ENEMY_FLY_LEFT:
                            case ENEMY_FLY_RIGHT:
                                enemies[enemyCounter * 16 + 0] = 1;//enable
                                enemies[enemyCounter * 16 + 1] = (byte)(enemyCounter*2);//set start counter 0 2 4 6 
                                enemies[enemyCounter * 16 + 2] = (byte)(i * 8);//x
                                enemies[enemyCounter * 16 + 3] = (byte)(j * 8);//y
                        
                                int mapAddr = 32768 + 32 * j + i - 512 - 512;
                                enemies[enemyCounter * 16 + 8] = (byte)(mapAddr % 256);//start addr LOW
                                enemies[enemyCounter * 16 + 9] = (byte)(mapAddr / 256);//start addr HIGH

                                //sinus pointer for fly's
                                enemies[enemyCounter * 16 + 10] = (byte)(enemyCounter * 64);

                                //type
                                enemies[enemyCounter * 16 + 11] = ENEMY_FLY;

                                //HARDWARE SPRITE LINK
                                enemies[enemyCounter * 16 + 13] = (byte)(enemyCounter*2);

                                //SPRITE IMAGE LINK
                                enemies[enemyCounter * 16 + 14] = (byte)((126-tile)*2+8);


                                //step
                                enemyCounter++;
                                if (enemyCounter > 3)
                                    enemyCounter = 0;

                                break;


                            case ENEMY_WALK_LEFT:
                            case ENEMY_WALK_RIGHT:
                                enemies[enemyCounter * 16 + 0] = 1;//enable
                                enemies[enemyCounter * 16 + 1] = (byte)(enemyCounter * 2);//set start counter 0 2 4 6 
                                enemies[enemyCounter * 16 + 2] = (byte)(i * 8);//x
                                enemies[enemyCounter * 16 + 3] = (byte)(j * 8);//y

                                mapAddr = 32768 + 32 * j + i - 512 - 512;
                                enemies[enemyCounter * 16 + 8] = (byte)(mapAddr % 256);//start addr LOW
                                enemies[enemyCounter * 16 + 9] = (byte)(mapAddr / 256);//start addr HIGH

                                //sinus pointer for fly's
                                enemies[enemyCounter * 16 + 10] = (byte)(enemyCounter * 64);

                                //type
                                enemies[enemyCounter * 16 + 11] = ENEMY_WALK;

                                //HARDWARE SPRITE LINK
                                enemies[enemyCounter * 16 + 13] = (byte)(enemyCounter * 2);

                                //SPRITE IMAGE LINK
                                enemies[enemyCounter * 16 + 14] = (byte)((126 - tile) * 2 + 8);


                                //step
                                enemyCounter++;
                                if (enemyCounter > 3)
                                    enemyCounter = 0;

                                break;

                                


                        }

                    }

                }
            }

            Console.WriteLine();

           
            

            File.WriteAllBytes(args[0]+".mapa", tileMap);
            File.WriteAllBytes(args[0]+".bgmapa", tileMapBackground);
            File.WriteAllBytes(args[0] + ".obj", mapObjects);
            File.WriteAllBytes(args[0] + ".eguns", eGunsObjects);
            File.WriteAllBytes(args[0] + ".enemies", enemies);

            //===================================================

            //                Console.ReadKey();

        }
            

        }
}
